/**
	$Id: JobExecutor.java 4512 2008-09-11 20:05:50Z jari $

	Copyright (C) 2006 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.clients.jobagent;

import net.sf.basedb.core.Job;
import net.sf.basedb.core.JobAgentSettings;
import net.sf.basedb.core.SessionControl;

/**
 	This interface must be implemented by all classes that want's to act
 	as a job executor. A job executor is responsible for executing a job
 	when the job agent reqeusts so. The job executor to use is specfied
 	by the <code>agent.executor.class</code> configuration setting.
 	<p>
 	Each agent only has one instance of a job executor, which means
 	that a job executor must be thread-safe and able to execute more than
 	one job at a time. It is not a good solution to put the <code>synchonrized</code>
 	keyword on the {@link #executeJob(SessionControl, Agent, Job, JobAgentSettings, Job.ExecutionTime)} 
 	method since it would limit the execution to one job at a time.
 	<p>
 	A job executor must provide a public no-argument constructor.

	@author nicklas
	@version 2.0
	@base.modified $Date: 2008-09-11 22:05:50 +0200 (to, 11 sep 2008) $
*/
public interface JobExecutor
{

	/**
		Initialize the job executor. This method is called once immediately
		after the object has been created.
		@param agent The agent 
	*/
	public void init(Agent agent);
	
	/**
		Execute the specified job. This method is called in a new thread
		created specifically for running that job. If the agent is shutting
		down it will send an interrupt signal to all job threads. It is up
		to the job executor to decide how to handle this. It may either decide
		to kill the job or let it continue as if nothing happened.
		<p>
		The job sent to this method has it's status set to {@link 
		net.sf.basedb.core.Job.Status#PREPARED}.
		The implementation of this method must change the status to either 
		{@link net.sf.basedb.core.Job.Status#DONE} or {@link 
		net.sf.basedb.core.Job.Status#ERROR}. If the status hasn't changed
		the job agent will set the status to signal an unknown error.
		
		@param sc A <code>SessionControl</code> where the owner of the job is
			logged in and the correct active project is set
		@param agent The agent that sent the request to execute the job
		@param job The job to execute
		@param settings The specific settings used for the plugin on this job agent
			or null if no settings has been specified
		@param usedSlot The slot that was used to execut the job
	*/
	public void executeJob(SessionControl sc, Agent agent, Job job, JobAgentSettings settings, 
		Job.ExecutionTime usedSlot);
	
	/**
		This method is called at when the agent is stopped or paused. No more
		execution requests will be sent to the current job executor, but
		running jobs should NOT be terminated due to this method beeing called.
	*/
	public void close();
	
}
