/*
  $Id: QuotaType.java 7308 2017-03-13 08:15:46Z nicklas $

	Copyright (C) 2005 Johan Enell, Nicklas Nordborg
	Copyright (C) 2006 Jari Häkkinen, Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

  BASE is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 3
  of the License, or (at your option) any later version.

  BASE is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.core;

import java.util.Date;

import net.sf.basedb.core.data.QuotaTypeData;


/**
	This class represents the different types a quota can have. 
	
	@author enell
	@version 2.0
	@base.modified $Date: 2017-03-13 09:15:46 +0100 (må, 13 mar 2017) $
*/
public class QuotaType
	extends BasicItem
	implements Nameable, SystemItem, Registered
{
	/**
		The type of item represented by this class.
		@see Item#QUOTATYPE
		@see #getType()
	*/
	public static final Item TYPE = Item.QUOTATYPE;
	
	/**
		The ID for the total quota.
	*/
	public static final String TOTAL = "net.sf.basedb.core.QuotaType.TOTAL";
	/**
		The ID for the file.
	*/
	public static final String FILE = "net.sf.basedb.core.QuotaType.FILE";
	/**
		The ID for the raw data.
	*/
	public static final String RAW_DATA = "net.sf.basedb.core.QuotaType.RAWDATA";
	/**
		The ID for experiments.
	*/
	public static final String EXPERIMENT = "net.sf.basedb.core.QuotaType.EXPERIMENT";
	
	/**
		Get a <code>QuotaType</code> object when you know the id.

		@param dc The <code>DbControl</code> which will be used for
			permission checking and database access.
		@param id The id of the item to load
		@return The <code>QuotaType</code> item
		@throws ItemNotFoundException This exception is thrown if an
			item with the specified ID is not found
		@throws PermissionDeniedException This exception is thrown if
			the logged in user doesn't have {@link Permission#READ READ}
			permission to the items
		@throws BaseException This exception is thrown if there is another
			error
	*/
	public static QuotaType getById(DbControl dc, int id)
		throws ItemNotFoundException, PermissionDeniedException, BaseException
	{
		QuotaType qt = dc.loadItem(QuotaType.class, id);
		if(qt == null)
		{
			throw new ItemNotFoundException("QuotaType[id="+id+"]");
		}
		return qt;
	}
	
	/**
		Get a query configured to retrieve <code>QuotaType</code>. All logged 
		in users have READ permission to all QuotaType items.
		@return An {@link ItemQuery} object
	*/
	public static ItemQuery<QuotaType> getQuery()
	{
		return new ItemQuery<QuotaType>(QuotaType.class);
	}

	/**
		Creates a new <code>QuotaType</code> item.

		@param data The data class for this item. 
	*/
	QuotaType(QuotaTypeData data)
	{
		super(data);
	}

	@Override
	QuotaTypeData getData()
	{
		return (QuotaTypeData)getBasicData();
	}
	/*
		From the Identifiable interface
		-------------------------------------------
	*/
	@Override
	public Item getType()
	{
		return TYPE;
	}
	// -------------------------------------------
	/*
		From the Nameable interface
		-------------------------------------------
	*/
	@Override
	public String getName()
	{
		return getData().getName();
	}
	@Override
	public void setName(String name)
		throws PermissionDeniedException, InvalidDataException
	{
		checkPermission(Permission.WRITE);
		NameableUtil.setName(getData(), name);
	}
	@Override
	public String getDescription()
	{
		return getData().getDescription();
	}
	@Override
	public void setDescription(String description)
		throws PermissionDeniedException, InvalidDataException
	{
		checkPermission(Permission.WRITE);
		NameableUtil.setDescription(getData(), description);
	}
	// -------------------------------------------
	/*
		From the SystemItem interface
		-------------------------------------------
	*/
	@Override
	public String getSystemId()
	{
		return getData().getSystemId();
	}
	@Override
	public boolean isSystemItem()
	{
		return getData().getSystemId() != null;
	}
	// -------------------------------------------
	/*
		From the Registered interface
		-------------------------------------------
	*/
	@Override
	public Date getEntryDate()
	{
		return DateUtil.copy(getData().getEntryDate());
	}
	@Override
	public void setEntryDate(Date entryDate)
	{
		if (isInDatabase()) 
		{
			throw new PermissionDeniedException("Not allowed to change entryDate of " + this);
		}
		getData().setEntryDate(DateUtil.setDateOrToday(entryDate));
	}
	// -------------------------------------------

	/*
		From the BasicItem class
		-------------------------------------------
	*/
	/**
		@return Allways return true.
	*/
	@Override
	public boolean isUsed()
	{
		return true;
	}
	/**
		The logged in user has read permission to the quota
		assigned to him/her. If this is a system item, delete
		and create permissions are denied.
		@throws BaseException This exception is thrown if there is any error
	*/
	@Override
	void initPermissions(int granted, int denied)
		throws BaseException
	{
		if (isSystemItem()) 
		{
			denied |= Permission.deny(Permission.DELETE, Permission.CREATE);
		}
		granted |= Permission.grant(Permission.READ);
		super.initPermissions(granted, denied);
	}
	// -------------------------------------------

	/**
		Check if items using quota from this quota type can be stored
		in the secondary location or not.
		@return <code>true</code> if the secondary location can be used, <code>false</code> otherwise
	*/
	public boolean hasSecondaryLocation()
	{
		return getData().getSecondaryLocation();
	}
}
