/*
	$Id: RawBioAssay.java 7441 2018-02-01 10:32:53Z nicklas $

	Copyright (C) 2005 Nicklas Nordborg
	Copyright (C) 2006 Jari Häkkinen, Nicklas Nordborg
	Copyright (C) 2007 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.core;

import net.sf.basedb.core.data.ArrayDesignData;
import net.sf.basedb.core.data.FeatureData;
import net.sf.basedb.core.data.ProtocolData;
import net.sf.basedb.core.data.RawBioAssayData;
import net.sf.basedb.core.data.RawData;
import net.sf.basedb.core.data.ReporterData;
import net.sf.basedb.core.data.SoftwareData;
import net.sf.basedb.core.data.SpotImagesData;
import net.sf.basedb.core.query.Restrictions;
import net.sf.basedb.core.query.Hql;
import net.sf.basedb.core.query.PermissionRestriction;
import net.sf.basedb.core.query.Dynamic;
import net.sf.basedb.core.query.Expressions;
import net.sf.basedb.core.signal.SignalException;
import net.sf.basedb.core.signal.ThreadSignalHandler;

import java.util.Collection;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;
import java.util.Set;
import java.util.HashSet;
import java.util.Collections;
import java.util.TreeSet;

import org.hibernate.metadata.ClassMetadata;

/**
	A rawbioassay represents the raw data generated from a single
	{@link DerivedBioAssay}. A raw bioassay must have a {@link RawDataType}
	which specifies the data columns generated by the {@link Software}.
	<p>
	Raw data is added by a {@link RawDataBatcher} object. You can get a 
	batcher for this raw bioassay by calling the {@link 
	#getRawDataBatcher(FeatureIdentificationMethod, Job)} method.

	@author Nicklas
	@version 2.0
	@base.modified $Date: 2018-02-01 11:32:53 +0100 (to, 01 feb 2018) $
*/
public class RawBioAssay 
	extends AnnotatedItem
	implements DiskConsumable, FileStoreEnabled, Registered, Listable
{
	/**
		The type of item represented by this class.
		@see Item#RAWBIOASSAY
		@see #getType()
	*/
	public static final Item TYPE = Item.RAWBIOASSAY;

	/**
		The maximum length of the name of a header that can be
		stored in the database. Check the name length against
		this value before calling {@link #setHeader(String, String)}
		to avoid exceptions.
	*/
	public static final int MAX_HEADER_NAME_LENGTH = RawBioAssayData.MAX_HEADER_NAME_LENGTH;
	
	/**
		The maximum length of the value of a header that can be
		stored in the database. Check the value length against
		this value before calling {@link #setHeader(String, String)}
		to avoid exceptions.
	*/
	public static final int MAX_HEADER_VALUE_LENGTH = RawBioAssayData.MAX_HEADER_VALUE_LENGTH;

	/**
		Create a new raw bioassay for a specific platform and/or raw data type. 
		If the platform is locked to a specific raw data type, the <code>rawDataType</code>
		parameter is ignored. If the platform isn't locked the raw data type
		is required and must be a database-based raw data type. Ie. 
		{@link RawDataType#isStoredInDb()} must return true.
		
		@param dc The <code>DbControl</code> which will be used for
			permission checking and database access
		@param platform The platform of the new raw bioassay, null is not allowed
		@param rawDataType The raw data type of the raw bioassay (required if
			the platform isn't locked to a specific raw data type) 
		@return The new <code>RawBioAssay</code> item
		@since 2.5
	*/
	public static RawBioAssay getNew(DbControl dc, Platform platform, RawDataType rawDataType)
	{
		RawBioAssay rba = dc.newItem(RawBioAssay.class);
		rba.setPlatform(platform, rawDataType);
		rba.setName("New raw bioassay");
		rba.getData().setEntryDate(new Date());
		return rba;
	}
	
	/**
		Create a new raw bioassay for a specific platform variant. 
		If the variant is locked to a specific raw data type, the <code>rawDataType</code>
		parameter is ignored. If the variant isn't locked the raw data type
		is required and must be a database-based raw data type. Ie. 
		{@link RawDataType#isStoredInDb()} must return true.

		
		@param dc The <code>DbControl</code> which will be used for
			permission checking and database access
		@param variant The platform variant of the new raw bioassay, null is not allowed
		@param rawDataType The raw data type of the raw bioassay (required if
			the platform isn't locked to a specific raw data type) 
		@return The new <code>RawBioAssay</code> item
		@since 2.5
	*/
	public static RawBioAssay getNew(DbControl dc, PlatformVariant variant, RawDataType rawDataType)
	{
		RawBioAssay rba = dc.newItem(RawBioAssay.class);
		rba.setVariant(variant, rawDataType);
		rba.setName("New raw bioassay");
		rba.getData().setEntryDate(new Date());
		return rba;	
	}

	/**
		Get a <code>RawBioAssay</code> object when you know the id.

		@param dc The <code>DbControl</code> which will be used for
			permission checking and database access.
		@param id The id of the item to load
		@return The <code>RawBioAssay</code> item
		@throws ItemNotFoundException If an item with the specified id is
			not found
		@throws PermissionDeniedException If the logged in user doesn't
			have read permission to the item
		@throws BaseException If there is another error
	*/
	public static RawBioAssay getById(DbControl dc, int id)
		throws ItemNotFoundException, PermissionDeniedException, BaseException
	{
		RawBioAssay rba = dc.loadItem(RawBioAssay.class, id);
		if (rba == null) throw new ItemNotFoundException("RawBioAssay[id=" + id + "]");
		return rba;
	}
	
	/**
		Get a query configured to retrieve raw bioassays.
		@return An {@link ItemQuery} object
	*/
	public static ItemQuery<RawBioAssay> getQuery() 
	{
		return new ItemQuery<RawBioAssay>(RawBioAssay.class);
	}
	
	private RawDataBatcher rawDataBatcher;
	
	RawBioAssay(RawBioAssayData data)
	{
		super(data);
	}

	@Override
	RawBioAssayData getData()
	{
		return (RawBioAssayData)getBasicData();
	}
	/*
		From the Identifiable interface
		-------------------------------------------
	*/
	@Override
	public Item getType()
	{
		return TYPE;
	}
	// -------------------------------------------
	/*
		From the DiskConsumable interface
		-------------------------------------------
	*/
	/**
		Always FALSE.
	*/
	@Override
	public boolean useDifference()
	{
		return false;
	}
	@Override
	public long getBytes()
	{
		return getData().getBytes();
	}
	/**
		Always 0.
	*/
	@Override
	public long getDifference()
	{
		return 0;
	}
	/**
		Always {@link QuotaType#RAW_DATA}.
	*/	
	@Override
	public String getQuotaTypeSystemId() 
	{
		return QuotaType.RAW_DATA;
	}
	/**
		Always {@link Location#PRIMARY}.
	*/
	@Override
	public Location getLocation()
	{
		return Location.PRIMARY;
	}
	// -------------------------------------------
	/**
		Get the parent derived bioassay and extract, if any.
	*/
	@Override
	public Set<Annotatable> getAnnotatableParents()
		throws BaseException
	{
		Set<Annotatable> annotatable = new HashSet<Annotatable>();
		RawBioAssayData data = getData();

		addAnnotatableParents(getDbControl(), annotatable, 
			data.getArrayDesign(), 
			data.getParentBioAssay(),
			data.getParentExtract(),
			data.getSoftware(),
			data.getProtocol()
		);
		return annotatable;
	}
	// -------------------------------------------
	/*
		From the FileStoreEnabled interface
		-------------------------------------------
	*/
	/**
		@since 2.5
	*/
	@Override
	public FileSet getFileSet()
	{
		DbControl dc = getDbControl();
		FileSet fs = dc.getItem(FileSet.class, getData().getFileSet(), this);
		if (fs == null)
		{
			fs = FileSet.getNew(dc, this);
			getData().setFileSet(fs.getData());
			dc.saveItemIf(this, fs, false);
		}
		return fs;
	}
	/**
		@since 2.5
	*/
	@Override
	public boolean hasFileSet()
	{
		return getData().getFileSet() != null;
	}
	/**
		@since 2.5
	*/
	@Override
	public Platform getPlatform()
	{
		return getDbControl().getItem(Platform.class, getData().getPlatform());
	}
	/**
		@since 2.5
	*/
	@Override
	public PlatformVariant getVariant()
	{
		return getDbControl().getItem(PlatformVariant.class, getData().getVariant());
	}
	/**
		Get the file set from the array design, if it exists.
		@since 2.5
	*/
	@Override
	public Collection<FileSet> getParentFileSets()
	{
		Collection<FileSet> parents = null;
		ArrayDesign design = getArrayDesign();
		if (design != null && design.hasFileSet())
		{
			parents = Collections.singleton(design.getFileSet());
		}
		return parents;
	}
	// -------------------------------------------
	/*
		From the Registered interface
		-------------------------------------------
	*/
	@Override
	public Date getEntryDate()
	{
		return DateUtil.copy(getData().getEntryDate());
	}
	@Override
	public void setEntryDate(Date entryDate)
	{
		if (isInDatabase()) 
		{
			throw new PermissionDeniedException("Not allowed to change entryDate of " + this);
		}
		getData().setEntryDate(DateUtil.setDateOrToday(entryDate));
	}
	// -------------------------------------------

	/*
		From the BasicItem class
		-------------------------------------------
	*/
	/**
		Check if:
		<ul>
		<li>An {@link Experiment} is using this raw bioassay
		<li>A {@link Transformation} is using this raw bio raw data type will also be updated. The platform can't be changed
		if raw data has been imported to the database.assay
		</ul>
	*/
	@Override
	public boolean isUsed() 
		throws BaseException
	{
		org.hibernate.Session session = getDbControl().getHibernateSession();
		org.hibernate.Query query = HibernateUtil.getPredefinedQuery(session, 
			"GET_EXPERIMENTS_FOR_RAWBIOASSAY", "count(*)");
		/*
			SELECT {1}
			FROM ExperimentData xp
			JOIN xp.rawBioAssays rba
			WHERE rba = :rawBioAssay
		*/
		query.setEntity("rawBioAssay", this.getData());
		boolean used = HibernateUtil.loadData(Long.class, query) > 0;
		if (!used)
		{
			query = HibernateUtil.getPredefinedQuery(session, 
				"GET_TRANSFORMATIONS_FOR_RAWBIOASSAY", "count(*)");
			/*
				SELECT {1}
				FROM TransformationData trf
				JOIN trf.rawSources rba
				WHERE rba = :rawBioAssay
			*/
			query.setEntity("rawBioAssay", this.getData());
			used = HibernateUtil.loadData(Long.class, query) > 0;
		}
		return used || super.isUsed();
	}
	/**
		Get all:
		<ul>
		<li>{@link Experiment}:s using this raw bioassay
		<li>{@link Transformation}:s using this raw bioassay
		</ul>
		@since 2.2
	*/
	@Override
	public Set<ItemProxy> getUsingItems()
	{
		Set<ItemProxy> using = super.getUsingItems();
		org.hibernate.Session session = getDbControl().getHibernateSession();
		
		// Experiments
		org.hibernate.Query query = HibernateUtil.getPredefinedQuery(session, 
			"GET_EXPERIMENTS_FOR_RAWBIOASSAY", "xp.id");
			/*
				SELECT {1}
				FROM ExperimentData xp
				JOIN xp.rawBioAssays rba
				WHERE rba = :rawBioAssay
			*/
		query.setEntity("rawBioAssay", this.getData());
		addUsingItems(using, Item.EXPERIMENT, query);
		
		// Transformations
		query = HibernateUtil.getPredefinedQuery(session, 
			"GET_TRANSFORMATIONS_FOR_RAWBIOASSAY", "trf.id");
			/*
				SELECT {1}
				FROM TransformationData trf
				JOIN trf.rawSources rba
				WHERE rba = :rawBioAssay
			*/
		query.setEntity("rawBioAssay", this.getData());
		addUsingItems(using, Item.TRANSFORMATION, query);
		return using;
	}
	/**
		Delete raw data if it exists.
	*/
	@Override
	void onBeforeCommit(Transactional.Action action)
		throws BaseException
	{
		super.onBeforeCommit(action);
		if (action == Transactional.Action.DELETE)
		{
			if (getRawDataType().isStoredInDb() && getNumDbSpots() > 0)
			{
				RawDataBatcher rdb = RawDataBatcher.getNew(getDbControl(), this, null);
				rdb.deleteAll();
			}
		}
	}
	/**
		Set software, protocol and array design from project default settings.
		@since 2.9
	*/
	@Override
	void setProjectDefaults(Project activeProject)
		throws BaseException
	{
		super.setProjectDefaults(activeProject);
		if (!softwareHasBeenSet)
		{
			SoftwareData software = 
				(SoftwareData)activeProject.findDefaultData(getDbControl(), Software.FEATURE_EXTRACTION, false);
			if (software != null) getData().setSoftware(software);
			softwareHasBeenSet = true;
		}
		if (!arrayDesignHasBeenSet)
		{
			ArrayDesignData design = 
				(ArrayDesignData)activeProject.findDefaultData(Item.ARRAYDESIGN, null, false);
			if (design != null) getData().setArrayDesign(design);
			arrayDesignHasBeenSet = true;
		}
		if (!protocolHasBeenSet)
		{
			ProtocolData protocol = 
				(ProtocolData)activeProject.findDefaultData(getDbControl(), Protocol.FEATURE_EXTRACTION, false);
			if (protocol != null) getData().setProtocol(protocol);
			protocolHasBeenSet = true;
		}
	}
	// -------------------------------------------

	/**
		Set the platform and raw data type of the raw bioassay. This method will 
		set the variant to null. Use {@link #setVariant(PlatformVariant, RawDataType)} if you 
		want to set a specific variant. If the platform is locked to a raw data type,
		the <code>rawDataType</code> parameter is ignored. If the platform isn't
		locked the raw data type must be a database-based raw data type. Ie. 
		{@link RawDataType#isStoredInDb()} must return true.

		@param platform The new platform, null is not allowes
		@param rawDataType The raw data type, required if the platform
			doesn't specify a raw data type
		@throws PermissionDeniedException If the logged in user doesn't have
			write permission
		@throws InvalidDataException If platform is null or if the raw data
			type is not properly specified
		@since 2.5
	*/
	public void setPlatform(Platform platform, RawDataType rawDataType)
	{
		checkPermission(Permission.WRITE);
		if (platform == null) throw new InvalidUseOfNullException("platform");
		if (getData().getSpots() > 0) 
		{
			throw new PermissionDeniedException("Not allowed to change the platform after raw data has been added");
		}
		RawDataType rdt = platform.getRawDataType();
		if (rdt == null) 
		{
			if (rawDataType == null)
			{
				throw new InvalidUseOfNullException("rawDataType must be specified for platform: " + 
					platform.getName());
			}
			if (!rawDataType.isStoredInDb())
			{
				throw new InvalidDataException("rawDataType can't store data in the database: " + 
					rawDataType.getName());
			}
			rdt = rawDataType;
		}
		getData().setPlatform(platform.getData());
		getData().setVariant(null);
		getData().setRawDataType(rdt.getId());
	}
	
	/**
		Set the platform and variant of the raw bioassay. This method will 
		automatically set the platform to {@link PlatformVariant#getPlatform()}.
		If the platform variant is locked to a raw data type, the raw data
		type is also updated. The platform variant can't be changed
		if raw data has been imported to the database.
	
		@param variant The new platform variant
	 	@param rawDataType The rawdata type for this platform variant. 
	 		Null is allowed if rawDataType already has been set. 
		@throws PermissionDeniedException If the logged in user doesn't have
			write permission
		@throws InvalidDataException If variant is null
		@since 2.5
	*/
	public void setVariant(PlatformVariant variant, RawDataType rawDataType)
	{
		checkPermission(Permission.WRITE);
		if (variant == null) throw new InvalidUseOfNullException("variant");
		if (getData().getSpots() > 0) 
		{
			throw new PermissionDeniedException("Not allowed to change the platform after raw data has been added");
		}
		RawDataType rdt = variant.getRawDataType();
		if (rdt == null) 
		{
			if (rawDataType == null)
			{
				throw new InvalidUseOfNullException("rawDataType must be specified for platform variant: " + 
					variant.getName());
			}
			if (!rawDataType.isStoredInDb())
			{
				throw new InvalidDataException("rawDataType can't store data in the database: " + 
					rawDataType.getName());
			}
			rdt = rawDataType;
		}
		getData().setVariant(variant.getData());
		getData().setPlatform(variant.getData().getPlatform());
		getData().setRawDataType(rdt.getId());
	}	
	
	/**
		Check if the platform/variant of this raw bioassay has the
		given external ID. 
		@param externalId The external ID to match
		@return TRUE if either the variant of platform matches the
			external ID
		@since 2.5
	*/
	public boolean isPlatform(String externalId)
	{
		if (externalId == null) return false;
		if (externalId.equals(getData().getPlatform().getExternalId())) return true;
		if (getData().getVariant() != null)
		{
			if (externalId.equals(getData().getVariant().getExternalId())) return true;
		}
		return false;
	}

	/**
		Check if this raw bioassay uses a file-only platform or variant.
		If the raw bioassay has a platform variant, the setting from the 
		variant is returned, otherwise the setting from the platform.
		@return TRUE if this raw bioassay use a file-only variant / platform
		@since 2.5
	*/
	public boolean isFileOnlyPlatform()
	{
		if (getData().getVariant() != null)
		{
			return getData().getVariant().isFileOnly();
		}
		else
		{
			return getData().getPlatform().isFileOnly();
		}
	}
	
	/**
		Get the raw data type this raw bioassay uses for the raw data.
		The raw data type cannot be changed once the raw bioassay has been created.
		@return A <code>RawDataType</code> object
	*/
	public RawDataType getRawDataType()
	{
		return RawDataTypes.getSafeRawDataType(getData().getRawDataType());
	}
	
	/**
		Set the job on this raw bioassay that was used to create or import
		data for it.

		@param job The job or null to clear the existing job
		@throws PermissionDeniedException If the logged in user doesn't have 
			write permission
		@since 3.11.3
	*/
	public void setJob(Job job)
		throws PermissionDeniedException
	{
		checkPermission(Permission.WRITE);
		if (job != null) job.checkPermission(Permission.USE);
		getData().setJob(job == null ? null : job.getData());
	}
	
	/**
		Get the job that created or imported data to this raw bioassay.
		@return A <code>Job</code> object, ot null if this information is
			not available
		@throws PermissionDeniedException If the logged in user doesn't have 
			read permission to the job
		@throws BaseException If there is another error
		@since 2.7
	*/
	public Job getJob()
		throws PermissionDeniedException, BaseException
	{
		return getDbControl().getItem(Job.class, getData().getJob());
	}
	
	/**
		Get the {@link DerivedBioAssay} this raw bioassay is created from. 

		@return The <code>DerivedBioAssay</code> item or null if not known
		@throws PermissionDeniedException If the logged in user doesn't have 
			read permission for the scanner
		@throws BaseException If there is another error.
		@since 3.0
	*/
	public DerivedBioAssay getParentBioAssay()
		throws PermissionDeniedException, BaseException
	{
		return getDbControl().getItem(DerivedBioAssay.class, getData().getParentBioAssay());
	}
	
	/**
		Set the {@link DerivedBioAssay} this raw bioassay is created from. 

		@param parent The parent bioassay or null if not known
		@throws PermissionDeniedException If the logged in user doesn't have 
			write permission for the raw bioassay use permission for the scan
		@throws BaseException If there is another error
		@since 3.0
	*/
	public void setParentBioAssay(DerivedBioAssay parent)
		throws PermissionDeniedException
	{
		checkPermission(Permission.WRITE);
		if (parent != null) parent.checkPermission(Permission.USE);
		getData().setParentBioAssay(parent == null ? null : parent.getData());
	}

	/**
		Get the {@link Extract} this raw bioassay is created from. 
	
		@return The <code>Extract</code> item or null if not known
		@throws PermissionDeniedException If the logged in user doesn't have 
			read permission for the extract
		@throws BaseException If there is another error.
		@since 3.0
	*/
	public Extract getParentExtract()
		throws PermissionDeniedException, BaseException
	{
		return getDbControl().getItem(Extract.class, getData().getParentExtract());
	}
	
	/**
		Set the {@link Extract} this raw bioassay is created from. 
	
		@param extract The parent extract or null if not known
		@throws PermissionDeniedException If the logged in user doesn't have 
			write permission for the raw bioassay use permission for the scan
		@throws BaseException If there is another error
		@since 3.0
	*/
	public void setParentExtract(Extract extract)
		throws PermissionDeniedException
	{
		checkPermission(Permission.WRITE);
		if (extract != null) extract.checkPermission(Permission.USE);
		getData().setParentExtract(extract == null ? null : extract.getData());
	}

	
	/**
		Get the {@link Software} used while analysing the scanned images.

		@return The <code>Software</code> item or null if not known
		@throws PermissionDeniedException If the logged in user doesn't have 
			read permission for the software
		@throws BaseException If there is another error.
	*/
	public Software getSoftware()
		throws PermissionDeniedException, BaseException
	{
		return getDbControl().getItem(Software.class, getData().getSoftware());
	}
	
	private boolean softwareHasBeenSet = false;
	/**
		Set the {@link Software} used while analysing the scanned images.

		@param software The software or null if not known
		@throws PermissionDeniedException If the logged in user doesn't have 
			write permission for the raw bioassay use permission for the scanner
		@throws BaseException If there is another error
	*/
	public void setSoftware(Software software)
		throws PermissionDeniedException
	{
		checkPermission(Permission.WRITE);
		if (software != null) software.checkPermission(Permission.USE);
		softwareHasBeenSet = true;
		getData().setSoftware(software == null ? null : software.getData());
	}

	/**
		Get the {@link Protocol} used while analysing the scanned images.

		@return The <code>Protocol</code> item or null if not known
		@throws PermissionDeniedException If the logged in user doesn't have 
			read permission for the protocol
		@throws BaseException If there is another error.
	*/
	@Override
	public Protocol getProtocol()
		throws PermissionDeniedException, BaseException
	{
		return getDbControl().getItem(Protocol.class, getData().getProtocol());
	}
	
	private boolean protocolHasBeenSet = false;
	/**
		Set the {@link Protocol} used while analysing the scanned images.

		@param protocol The protocol or null if not known
		@throws PermissionDeniedException If the logged in user doesn't have 
			write permission for the raw bioassay use permission for the protocol
		@throws BaseException If there is another error
	*/
	public void setProtocol(Protocol protocol)
		throws PermissionDeniedException
	{
		checkPermission(Permission.WRITE);
		if (protocol != null) protocol.checkPermission(Permission.USE);
		protocolHasBeenSet = true;
		getData().setProtocol(protocol == null ? null : protocol.getData());
	}

	/**
		Get the {@link ArrayDesign} this raw data uses. This should normally
		be the same as the array design reached via the hybridization, but
		doesn't have to be.

		@return The <code>ArrayDesign</code> item or null if not known
		@throws PermissionDeniedException If the logged in user doesn't have 
			read permission for the array design
		@throws BaseException If there is another error.
	*/
	public ArrayDesign getArrayDesign()
		throws PermissionDeniedException, BaseException
	{
		return getDbControl().getItem(ArrayDesign.class, getData().getArrayDesign());
	}
	
	private boolean arrayDesignHasBeenSet = false;
	/**
		Set the {@link ArrayDesign} this raw data uses. The array design can't be 
		changed with this method once raw data spots has been added. Use
		the {@link #updateArrayDesign(ArrayDesign, FeatureIdentificationMethod, ProgressReporter)} to
		do that.
	
		@param arrayDesign The <code>ArrayDesign</code> item or null if not known
		@throws PermissionDeniedException If the logged in user doesn't have 
			write permission for the raw bioassay use permission for the array design
			or spots has already been added
		@throws BaseException If there is another error
	*/
	public void setArrayDesign(ArrayDesign arrayDesign)
		throws PermissionDeniedException
	{
		checkPermission(Permission.WRITE);
		if (arrayDesign != null) arrayDesign.checkPermission(Permission.USE);
		if (getData().getHasData()) 
		{
			throw new PermissionDeniedException("Not allowed to change the array design after raw data has been added");
		}
		arrayDesignHasBeenSet = true;
		getData().setArrayDesign(arrayDesign == null ? null : arrayDesign.getData());
	}

	/**
		Checks if the given array design is the same as already set
		on this raw bioassay.
		@since 3.0
	*/
	public boolean isSameArrayDesign(ArrayDesign design)
	{
		if (design == null && getData().getArrayDesign() == null) return true;
		if (design != null && design.getData().equals(getData().getArrayDesign())) return true;
		return false;
	}
	
	/**
		Set the {@link ArrayDesign} this raw data uses. If data has already been
		imported to the raw bioassay, the reporters for each position on the 
		new array design must match the positions of reporters already present in 
		the raw bioassay.
	
		@param arrayDesign The <code>ArrayDesign</code> item or null if not known
		@param fiMethod The method to use for identifying features on the
			new array design. Use null to use the method choosen by the new array design.
			The {@link FeatureIdentificationMethod#FEATURE_ID} will not work unless the
			raw bioassay is already connected to an array design which has features with
			external ID:s
		@param progress An optional progress reporter
		@throws PermissionDeniedException If the logged in user doesn't have 
			write permission for the raw bioassay use permission for the array design
		@throws InvalidDataException If the features of the new array design doesn't
			match the raw data
		@throws BaseException If there is another error
		@since 2.6
	*/
	public void updateArrayDesign(ArrayDesign arrayDesign, FeatureIdentificationMethod fiMethod, ProgressReporter progress)
		throws PermissionDeniedException, InvalidDataException
	{
		checkPermission(Permission.WRITE);
		if (arrayDesign != null) arrayDesign.checkPermission(Permission.USE);
		
		// No need to validate if there is no change
		if (isSameArrayDesign(arrayDesign)) return;
		
		// Re-validate data in database if we already have db data
		if (getData().getSpots() > 0 && arrayDesign != null) 
		{
			validateFeatures(getDbControl(), arrayDesign, fiMethod, true, progress);
		}
		getData().setArrayDesign(arrayDesign == null ? null : arrayDesign.getData());

		// Re-validate files
		if (hasFileSet()) getFileSet().validate(getDbControl());
	}	

	/**
		Check if this raw bioassay is connected to an array design or not.
		@return TRUE if it has an array design, FALSE otherwise
	*/
	public boolean hasArrayDesign()
	{
		return getData().getArrayDesign() != null;
	}
	
	/**
		Utility method that either returns the number of db spots
		or the number of file spots. If there is spot data in
		the database, this number is returned, otherwise the number
		of file spots is returned.
		@return The number of spots
	*/
	public int getSpots()
	{
		int dbSpots = getNumDbSpots();
		return dbSpots > 0 ? dbSpots : getNumFileSpots();
	}
	
	/**
		Get the number of spots on this raw bioassay that are
		stored in the database.
		@return The number of spots in the database
		@since 2.5
	*/
	public int getNumDbSpots()
	{
		return getData().getSpots();
	}
	
	/**
		Get the number of spots on this raw bioassay that are
		stored inside files.
		@return The number of spots in files
		@since 2.5
	*/
	public int getNumFileSpots()
	{
		return getData().getNumFileSpots();
	}
	
	/**
		Set the number of spots that are stored in files.
		@param numSpots The number of spots
		@since 2.5
	*/
	public void setNumFileSpots(int numSpots)
	{
		checkPermission(Permission.WRITE);
		getData().setNumFileSpots(numSpots);
		getData().setHasData(numSpots > 0 || getNumDbSpots() > 0);
	}
	
	
	/**
		If raw data has been added to this raw bioassay or not. Note that this method
		checks both data in the database and data in the form of files.
	*/
	public boolean hasData()
	{
		return getData().getHasData();
	}
	
	/**
		Get the {@link SpotImages} item used to generate spot images
		for this raw bioassay. If no <code>SpotImages</code> exists
		a new one is created, and is automatically saved to the database (if 
		commit is called).

		@return The <code>SpotImages</code> item
	 	@throws PermissionDeniedException If logged in user doesn't have read permission to the spot images. 
	 	@throws BaseException If there is some kind of error.
		@see #hasSpotImages()
	*/
	public SpotImages getSpotImages()
		throws PermissionDeniedException, BaseException
	{
		SpotImagesData spi = getData().getSpotImages();
		SpotImages spotImages = null;
		if (spi == null)
		{
			spotImages = SpotImages.getNew(getDbControl(), this);
			getDbControl().saveItemIf(this, spotImages, false);
		}
		else
		{
			spotImages = getDbControl().getItem(SpotImages.class, spi);
		}
		return spotImages;
	}
	
	/**
		Check if spot images exists for this raw bioassay or not.
		@see #getSpotImages()
	*/
	public boolean hasSpotImages()
	{
		return getData().getSpotImages() != null;
	}

	/**
		Get the value for the raw data header with the specified name.
		@param name The name of the header
		@return The value of the header or null if no header with that
			name exists
	*/
	public String getHeader(String name)
	{
		return getData().getHeaders().get(name);
	}
	
	/**
		Set the value for a raw data header.
		@param name The name of the header
		@param value The value of the header
		@throws PermissionDeniedException If the logged in user doesn't
			have write permission or if spots already has been added to the
			raw bioassay
		@throws InvalidDataException If the name or value is null or too long
	*/
	public void setHeader(String name, String value)
		throws PermissionDeniedException, InvalidDataException
	{
		checkPermission(Permission.WRITE);
		if (name == null) throw new InvalidUseOfNullException("header.name");
		if (name.length() > MAX_HEADER_NAME_LENGTH) 
		{
			throw new StringTooLongException("header.name", name, MAX_HEADER_NAME_LENGTH);
		}
		if (value == null) throw new InvalidUseOfNullException("header.value["+name+"]");
		if (value.length() > MAX_HEADER_VALUE_LENGTH) 
		{
			throw new StringTooLongException("header.value["+name+"]", value, MAX_HEADER_VALUE_LENGTH);
		}
		getData().getHeaders().put(name, value);
	}
	
	/**
		Remove all headers from the raw bioassay.
		@since 2.5
	*/
	public void removeHeaders()
	{
		checkPermission(Permission.WRITE);
		getData().getHeaders().clear();
	}

	/**
		Remove a specific header from the raw bioassay.
		@param name The name of the header
		@since 2.5
	*/
	public void removeHeader(String name)
	{
		checkPermission(Permission.WRITE);
		getData().getHeaders().remove(name);
	}

	/**
		Get the names of all raw data headers. The set is sorted by name.
		@return A set containing the header names or an empty set if no
			headers have been defined
	*/
	public Set<String> getHeaderNames()
	{
		return Collections.unmodifiableSet(new TreeSet<String>(getData().getHeaders().keySet()));
	}
	
	/**
		Get a query that returns all experiments where this raw bioassay is used.
		@return An {@link ItemQuery} object
	*/
	public ItemQuery<Experiment> getExperiments()
	{
		ItemQuery<Experiment> query = Experiment.getQuery();
		query.joinPermanent(Hql.innerJoin("rawBioAssays", Item.RAWBIOASSAY.getAlias()));
		query.restrictPermanent(
			Restrictions.eq(
				Hql.alias(Item.RAWBIOASSAY.getAlias()), 
				Hql.entity(this)
			)
		);
		return query;
	}
	
	/**
		Get a raw data batcher object which can be used to add raw data to
		the raw bioassay.
		
		@param fiMethod The method to use for identifying features, or null to
			use the method choosen by the array design. This parameter has no meaning
			if the raw bioassay is not connected to an array design or if the array
			design has no features.
		@param job The job that is currently running, or null if the raw data import
			is not running as a job
		@return A <code>RawDataBatcher</code> object. If this method is called multiple
			times the <code>fiMethod</code> and <code>job</code> parameter is ignored 
			and the same batcher as before is returned
		@throws PermissionDeniedException If raw data has already been added
			or the logged in user doesn't have write permission
		@throws BaseException If there is another error
		@since 2.7
	*/
	public RawDataBatcher getRawDataBatcher(FeatureIdentificationMethod fiMethod, Job job)
		throws PermissionDeniedException, BaseException
	{
		checkPermission(Permission.WRITE);
		if (getNumDbSpots() > 0) 
		{
			throw new PermissionDeniedException("Raw data has already been added to "+this);
		}
		if (rawDataBatcher == null)
		{
			rawDataBatcher = RawDataBatcher.getNew(getDbControl(), this, fiMethod);
			if (job != null) getData().setJob(job.getData());
		}
		return rawDataBatcher;
	}
	
	/**
		Delete the raw data that has been added to the database for this
		raw bioassay. You should not call this method in the same transaction as 
		adding raw data with the batcher from {@link #getRawDataBatcher(FeatureIdentificationMethod, Job)}.

		@throws PermissionDeniedException If the logged in user doesn't have
			write permission or a {@link RawDataBatcher} has been fetched
			using the {@link #getRawDataBatcher(FeatureIdentificationMethod, Job)} 
			method or the raw bioassay is used by some experiment
		@throws BaseException If there is another error
	*/
	public void deleteRawData()
		throws PermissionDeniedException, BaseException
	{
		checkPermission(Permission.WRITE);
		if (rawDataBatcher != null)
		{
			throw new PermissionDeniedException("Batching is in progress for raw bioassay: "+this);
		}
		if (isUsed())
		{
			throw new PermissionDeniedException("The raw bioassay is used in one or more experiments: "+this);
		}
		if (getNumDbSpots() > 0)
		{
			RawDataBatcher rdb = RawDataBatcher.getNew(getDbControl(), this, null);
			getData().setJob(null);
			rdb.deleteAll();
		}
	}
	
	/**
		Get a raw data spot when you know the id. NOTE! This method may return an uninitialized
		proxy. If an initialized object is required use {@link #getRawDataById(int, boolean)}
		instead.
		
		@param id The id of the spot
	 	@return An {@link net.sf.basedb.core.data.RawData} object 
	 	@throws ItemNotFoundException If no raw data item was found for the given id.
	 	@throws BaseException If there some other error.
	*/
	public RawData getRawDataById(int id)
		throws ItemNotFoundException, BaseException
	{
		return getRawDataById(id, false);
	}
	
	/**
		Get a raw data spot when you know the id.
		
		@param id The id of the spot
		@param forceInitialization If TRUE the raw data and associated feature and 
			reporter are sure to be initialized
	 	@return An {@link net.sf.basedb.core.data.RawData} object 
	 	@throws ItemNotFoundException If no raw data item was found for the given id.
	 	@throws BaseException If there some other error.
	 	@since 2.15
	*/
	public RawData getRawDataById(int id, boolean forceInitialization)
		throws ItemNotFoundException, BaseException
	{
		RawDataType rdt = getRawDataType();
		if (!rdt.isStoredInDb())
		{
			throw new BaseException("Raw data for raw data type '" + rdt + "' is not stored in the database.");
		}
		org.hibernate.Session session = getDbControl().getHibernateSession();
		RawData rd = HibernateUtil.loadData(session, RawData.class, rdt.getEntityName(), id);
		if (rd == null) throw new ItemNotFoundException("RawData[id = " + id + "]");
		if (forceInitialization)
		{
			org.hibernate.Hibernate.initialize(rd);
			org.hibernate.Hibernate.initialize(rd.getReporter());
			org.hibernate.Hibernate.initialize(rd.getFeature());
		}
		HibernateUtil.evictData(session, rd);
		return rd;
	}
	
	/**
		Get a query that returns the raw data of this raw bioassay.
		@return A {@link DataQuery} object
	*/
	public DataQuery<RawData> getRawData()
	{
		RawDataType rdt = getRawDataType();
		if (!rdt.isStoredInDb())
		{
			throw new BaseException("Raw data for raw data type '" + rdt + "' is not stored in the database.");
		}
		DataQuery<RawData> query = new DataQuery<RawData>(RawData.class, rdt.getEntityName());
		query.restrictPermanent(
			Restrictions.eq(
				Hql.property("rawBioAssay"),
				Hql.entity(this)
			)
		);
		return query;
	}
	
	/**
		Get a query that returns the reporters on this raw bioassay.
		The query is rooted at the raw data, which means that:
		<ul>
		<li>All filters, sorting, etc. that works for {@link #getRawData()} can be 
			used without modifications on this query.
		<li>The same reporter may be returned multiple times if it is present on multiple
			spots.
		<li>'Null' values may be returned if there are spots without a reporter
		</ul>
		
		@return A {@link DataQuery} object
		@since 2.8	
	*/
	public DataQuery<ReporterData> getReporters()
	{
		RawDataType rdt = getRawDataType();
		if (!rdt.isStoredInDb())
		{
			throw new BaseException("Raw data for raw data type '" + rdt + "' is not stored in the database.");
		}
		DataQuery<ReporterData> query = 
			new DataQuery<ReporterData>(ReporterData.class, Item.RAWDATA, rdt.getEntityName(), "reporter");
		// We allow reporters without READ permission unless it has been explicitely DENIED
		query.restrictPermanent(new PermissionRestriction(Permission.DENIED, Item.REPORTER));
		query.restrictPermanent(
			Restrictions.eq(
				Hql.property("rawBioAssay"),
				Hql.entity(this)
			)
		);
		return query;
	}
	
	/**
		Get a dynamic query that returns the raw data of this raw bioassay.
		@return A {@link DynamicRawDataQuery} object
	*/
	public DynamicRawDataQuery getDynamicRawData()
	{
		DynamicRawDataQuery query = new DynamicRawDataQuery(this);
		query.restrictPermanent(
			Restrictions.eq(
				Dynamic.rawData("rawBioAssay"),
				Expressions.integer(this.getId())
			)
		);
		return query;
	}

	/**
		Validate that the features/reporters of the raw data matches the 
		features/reporter on the array design.
		@param dc The DbControl to use for database access
		@param design The array design to validate against
		@param fiMethod The method to use for identifying features, or null
			to use the method choosen by the array design
		@param update If the raw data should be linked to the new feature or not
		@param progress An optional progress reporter
		@since 2.6
	*/
	private void validateFeatures(DbControl dc, ArrayDesign design, FeatureIdentificationMethod fiMethod, 
		boolean update, ProgressReporter progress)
	{
		if (design.getNumDbFeatures() > 0)
		{
			if (fiMethod == null) fiMethod = design.getFeatureIdentificationMethod();
			if (fiMethod == FeatureIdentificationMethod.NONE) fiMethod = FeatureIdentificationMethod.COORDINATES;
			
			if (progress != null) progress.display(5, "Loading features from array design...");
			
			// Hibernate metadata for setting new features
			ClassMetadata metaData = HibernateUtil.getClassMetadata(getRawDataType().getEntityName());
			org.hibernate.Session session = dc.getHibernateSession();
			String entityName = getRawDataType().getEntityName();
			
			int numFeatures = design.getNumDbFeatures();
			Map<Object, FeatureData> features = new HashMap<Object, FeatureData>(numFeatures);
			org.hibernate.Query query = HibernateUtil.getPredefinedQuery(dc.getStatelessSession(), "PRELOAD_FEATURES");
			/*
				SELECT f 
				FROM FeatureData f 
				JOIN FETCH f.arrayDesignBlock b 
				LEFT JOIN FETCH f.reporter 
				WHERE b.arrayDesign = :arrayDesign
			*/
			query.setInteger("arrayDesign", design.getId());
			ScrollIterator<FeatureData> si = null;
			try
			{
				si = QueryExecutor.loadIterator(FeatureData.class, query, 
					dc.getStatelessSession(), dc.getSessionControl());
			}
			catch (InterruptedException ex)
			{
				throw new SignalException("Aborted by user", ex);
			}
			while (si.hasNext())
			{
				ThreadSignalHandler.checkInterrupted();
				FeatureData f = si.next();
				Object featureId = fiMethod.getIdentifier(f);
				if (features.put(featureId, f) != null)
				{
					throw new InvalidDataException("Can't identify features with " + fiMethod + 
						". Found duplicate feature: " + featureId);
				}
			}
			si.close();
			
			// Load raw data
			if (progress != null) progress.display(10, "Loading raw data from raw bioassay...");
			DataQuery<RawData> rawQuery = getRawData();
			rawQuery.join(Hql.leftJoin(null, "reporter", "rep", null, true));
			rawQuery.join(Hql.leftJoin(null, "feature", "feat", null, true));

			DataResultIterator<RawData> result = rawQuery.iterate(dc);
			long numValidated = 0;
			int numTotal = getNumDbSpots();
			while (result.hasNext())
			{
				ThreadSignalHandler.checkInterrupted();
				RawData raw = result.next();

				// Get the feature identifier for the current raw data
				FeatureData oldFeature = raw.getFeature();
				String externalFeatureId = oldFeature == null ? null : oldFeature.getExternalId();
				Object rawFeatureId = fiMethod.getIdentifier(raw, externalFeatureId);
				
				// Find the new feature with the same identifier
				FeatureData newFeature = features.get(rawFeatureId);
				if (newFeature == null)
				{
					throw new InvalidDataException("Array design '" + design.getName() +
						"' is missing a feature for " + fiMethod + "=" + rawFeatureId);
				}
				
				// Check that reporters are the same
				ReporterData rawReporter = raw.getReporter();
				ReporterData featureReporter = newFeature.getReporter();
				if (rawReporter == null)
				{
					if (featureReporter != null) 
					{
						throw new InvalidDataException("Reporter mismatch at " +	fiMethod + "=" + rawFeatureId + 
							"; raw reporter=none, feature reporter=" + featureReporter.getExternalId());
					}
				}
				else
				{
					if (featureReporter == null)
					{
						throw new InvalidDataException("Reporter mismatch at " + fiMethod + "=" + rawFeatureId +
							"; raw reporter=" + rawReporter.getExternalId() + ", feature reporter=none");
					}
					else if (rawReporter.getId() != featureReporter.getId())
					{
						throw new InvalidDataException("Reporter mismatch at " + fiMethod + "=" + rawFeatureId +
							"; raw reporter=" + rawReporter.getExternalId() + 
							", feature reporter=" + featureReporter.getExternalId());
					}
				}
		
				// All is ok, re-assign raw data to the new feature
				if (update)
				{
					metaData.setPropertyValue(raw, "feature", newFeature);
					raw.setPosition(newFeature.getPosition());
					session.update(entityName, raw);
				}
				
				// Report progress
				numValidated++;
				if (progress != null && numValidated % 100 == 0)
				{
					int percent = 10 + ((int)(90L * numValidated) / numTotal);
					progress.display(percent, "Validated " + numValidated + " of " + numTotal + " spots.");
				}
			}
		}
	}
}
