/*
	$Id: Request.java 7321 2017-04-05 11:27:31Z nicklas $

	Copyright (C) 2005 Samuel Andersson, Johan Enell, Nicklas Nordborg
	Copyright (C) 2006 Jari Häkkinen, Nicklas Nordborg, Martin Svensson

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.core.plugin;

import java.io.Serializable;
import java.util.List;

import net.sf.basedb.core.Job;
import net.sf.basedb.core.ParameterException;
import net.sf.basedb.core.ProgressReporter;

/**
	Pass request information to a plugin. Objects implementing
	this interface are used by the core to pass information to a
	plugin in the {@link Plugin#run(Request, Response, ProgressReporter)}
	method. Commands are used to tell the plugin what kind of information
	the user want to pass to the plugin. 
	<BR>
	There are three predefind request commands that all plugins understands,
	those three are {@link Request#COMMAND_CONFIGURE_JOB}, 
	{@link Request#COMMAND_CONFIGURE_PLUGIN} and
	{@link Request#COMMAND_EXECUTE COMMAND_EXECUTE}. The configuration setup starts by giving the
	plugin the command {@link Request#COMMAND_CONFIGURE_PLUGIN}
	and it will be saved in the {@link net.sf.basedb.core.PluginConfiguration}
	that instantiated the plugin. The starting command for setting the job parameters are the
	{@link Request#COMMAND_CONFIGURE_PLUGIN} command.
	<p>
	After the request have been invoked the plugin will respond with a {@link Response}. 
	The status of the respond will tell the {@link net.sf.basedb.core.Job} how to proceed.
	Plugin defined commands are not allowed to use strings that begins with
	an underscore("_"). Strings starting with an underscore is reserved for the
	Base 2 system.

	@author Nicklas, Samuel, Enell
	@version 2.0
	@base.modified $Date$
*/
public interface Request
{
	/**
		This should be the first command to use when setting
		the plugin's configuration.
	*/
	public static final String COMMAND_CONFIGURE_PLUGIN = "_config_plugin";

	/**
		This should be the first command to use when setting
		the plugin's job parameters.
	*/
	public static final String COMMAND_CONFIGURE_JOB = "_config_job";

	/**
		This command will tell the plugin to start its execution.
	*/
	public static final String COMMAND_EXECUTE = "_exec";

	/**
		The command the plugin should execute. This value is one
		of the predefined constants above, or some other value
		that is supported by the specific plugin.
	*/
	public String getCommand();

	/**
		Get state information that was previously saved by
		{@link Response#setPause(String, String, java.io.Serializable)}
		or {@link Response#setContinue(String, java.io.Serializable)}.
		@return The state information or null
		@since 3.11
	*/
	public Serializable getState();
	
	/**
		Get the values for the parameter with the specified name.

		@param name The name of the parameter
		@return An object with the value, or null if no value
			is given
	*/
	public List<?> getParameterValues(String name);
	
	/**
		Get the value for the parameter with the specified name.

		@param name The name of the parameter
		@return An object with the value, or null if no value
			is given
	 	@throws ParameterException If getting the value fails. 
	*/
	public Object getParameterValue(String name)
		throws ParameterException;
	
	/**
		If this plugin has been allowed immediate execution or not. If this is true
		the plugin may end a job configuration sequence with {@link 
		Response#setExecuteImmediately(String, Job.ExecutionTime, boolean)}
		or, if the plugin is a {@link ImmediateDownloadExporter}, 
		{@link Response#setDownloadImmediately(String, Job.ExecutionTime, boolean)}. 
		@return TRUE if it is allowed, FALSE otherwise
		@since 2.2
	*/
	public boolean isAllowedImmediateExecution();
	
}
