/**
	$Id $

	Copyright (C) 2015 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.core.snapshot;

import java.util.Set;

import net.sf.basedb.core.AnnotationType;
import net.sf.basedb.core.DbControl;
import net.sf.basedb.util.filter.Filter;

/**
	Filter implementation that filters away annotations that
	have an annotation type with inheritance disabled. A parent
	filter may be specified with {@link #setParentFilter(Filter)}
	which is checked before checking the annotation type. This
	filter will only accept an annotatin if the parent filter accepts
	it and if the annotation type allows inheritance.

	@author Nicklas
	@since 3.5
*/
public class DisabledInheritanceFilter
	implements Filter<AnnotationSnapshot>
{

	private final Set<Integer> disabled;
	private Filter<? super AnnotationSnapshot> parent;
	
	/**
		Create a new filter. It will load all current annotation types with
		disabled inheritance and store internally.
		@see AnnotationType#getAllAnnotationTypesWithInheritanceDisabled(DbControl)
	*/
	public DisabledInheritanceFilter(DbControl dc)
	{
		this.disabled = AnnotationType.getAllAnnotationTypesWithInheritanceDisabled(dc);
	}
	
	/**
		Create a clone of another filter instance. Use this method to re-use the
		loaded annotation types when there is a risk of thread issues by using the
		root filter together with the {@link #setParentFilter(Filter)} method.
	*/
	public DisabledInheritanceFilter(DisabledInheritanceFilter other) 
	{
		this.disabled = other.disabled;
	}
	
	/*
		From the Filter interface
		-------------------------
	*/
	@Override
	public boolean evaluate(AnnotationSnapshot snapshot)
	{
		return (parent == null || parent.evaluate(snapshot)) && !disabled.contains(snapshot.getAnnotationTypeId());
	}
	// --------------------------

	/**
		Set a parent filter that must also be passed in order for the
		this filter to pass an annotation.
	*/
	public void setParentFilter(Filter<? super AnnotationSnapshot> parent)
	{
		this.parent = parent;
	}

}
