/**
	$Id: SimpleDynamicField.java 5385 2010-08-13 10:48:17Z nicklas $

	Copyright (C) 2009 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.util.export.spotdata;

import net.sf.basedb.core.BioAssaySet;
import net.sf.basedb.core.DbControl;
import net.sf.basedb.core.DynamicQuery;
import net.sf.basedb.core.Formula;
import net.sf.basedb.core.IntensityTransform;
import net.sf.basedb.core.Type;
import net.sf.basedb.core.query.Expression;
import net.sf.basedb.util.EqualsHelper;
import net.sf.basedb.util.formatter.Formatter;

/**
	Represents an exported data field. The object consists
	of a predefined query expression that can be used in the 
	export query to select the data that is going to be exported, and
	a title that is exported as part of the table header. One 
	instance of this class is considered to be equal to another if 
	they are configured with the same expression and title.

	@author Nicklas
	@version 2.12
	@base.modified $Date: 2010-08-13 12:48:17 +0200 (fr, 13 aug 2010) $
*/
public class SimpleDynamicField
	implements DynamicField
{

	private Expression expression;
	private Formula.AverageMethod averageMethod;
	private Formatter<?> formatter;
	private String title;
	private Type type;
	
	/**
		Create a new object. Before it can be used it must be configured.
	*/
	public SimpleDynamicField()
	{}
		
	/*
		From the DynamicField interface
		-------------------------------
	*/
	@Override
	public String getTitle()
	{
		return title;
	}
	/**
		Returns the configured expression. If 'forAverage' is TRUE the 
		confgured {@link net.sf.basedb.core.Formula.AverageMethod} is allowed to transform
		the expression. If no average method has been configured the 
		average method specified by the {@link IntensityTransform} from
		the 'source' bioassay set is used.
	*/
	@Override
	public Expression getExpression(DbControl dc, DynamicQuery query, BioAssaySet source, boolean forAverage)
	{
		Expression e = getExpression();
		if (forAverage && e != null)
		{
			Formula.AverageMethod avg = averageMethod == null ?
					source.getIntensityTransform().getAverageMethod() : averageMethod;
			e = avg.getAverageExpression(e);
		}
		return e;
	}
	@Override
	public Type getType()
	{
		return type;
	}
	@Override
	public Formatter<?> getFormatter()
	{
		return formatter;
	}
	// --------------------------------------------

	/**
		A dynamic field is equal to another field if it uses the same
		expression and has the same title. 
		@since 2.15
	*/
	@Override
	public boolean equals(Object obj)
	{
		if (this == obj) return true;
		if (!(obj instanceof SimpleDynamicField)) return false;
		SimpleDynamicField other = (SimpleDynamicField)obj;
		return EqualsHelper.equals(this.getExpression(), other.getExpression()) 
			&& EqualsHelper.equals(this.getTitle(), other.getTitle());
	}
	
	/**
		@since 2.15
	*/
	@Override
	public int hashCode()
	{
		return 13 * EqualsHelper.hashCode(getExpression()) + EqualsHelper.hashCode(getTitle());
	}
	/**
	 	@since 2.15
	*/
	@Override
	public String toString()
	{
		return "SimpleDynamicField(title=" + getTitle() + "; expression=" + getExpression() + ")";
	}

	/**
	 	Get the configured express.
	*/
	public Expression getExpression()
	{
		return expression;
	}
	
	/**
		Set the expression used to select data in the query.
	*/
	public void setExpression(Expression expression)
	{
		this.expression = expression;
	}

	/**
		Set the column header title.
	*/
	public void setTitle(String title)
	{
		this.title = title;
	}
	
	/**
		Set the data type of the exported field.
		@since 2.15
	*/
	public void setType(Type type)
	{
		this.type = type;
	}
	
	/**
		Get the average method to use if the exporter is exporting
		averaged data. If no average method has been configured, the
		everage method specified by the {@link IntensityTransform}
		of the source bioassay set is used.
	*/
	public Formula.AverageMethod getAverageMethod()
	{
		return averageMethod;
	}
	
	/**
		Set the average method to use when exporting averaged data.
	*/
	public void setAverageMethod(Formula.AverageMethod averageMethod)
	{
		this.averageMethod = averageMethod;
	}

	/**
		Set a formatter to use for formatting the exported values.
	*/
	public void setFormatter(Formatter<?> formatter)
	{
		this.formatter = formatter;
	}
	
}
