/**
	$Id$

	Copyright (C) 2011 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.util.extensions.manager.processor;

import net.sf.basedb.util.extensions.manager.ExtensionsFile;
import net.sf.basedb.util.extensions.manager.ExtensionsFileProcessor;
import net.sf.basedb.util.extensions.manager.ExtensionsManager;
import net.sf.basedb.util.extensions.manager.ExtensionsFile.WriteableExtensionsFile;
import net.sf.basedb.util.extensions.manager.ProcessResults;

/**
	File processor implementation that reset the error flag, class loader, etc.
	for all extension files. This is usually needed to be able to
	re-install extensions that has been updated or that had an error that 
	has now been fixed. This class replaces the {@link ResetErrorFlagProcessor}.

	@author Nicklas
	@since 3.10
	@base.modified $Date$
*/
public class ResetProcessor
	implements ExtensionsFileProcessor
{

	private static final org.slf4j.Logger log = 
		org.slf4j.LoggerFactory.getLogger(ResetProcessor.class);
	
	private final ProcessResults results;
	private boolean resetErrorFlag;
	private boolean resetClassLoader;
	
	private int numFiles;
	
	/**
		Create a new processor instance using default
		settings.
	*/
	public ResetProcessor(ProcessResults results)
	{
		this.results = results;
		this.resetClassLoader = true;
		this.resetErrorFlag = true;
	}
	
	/**
		This flag controls if the class loader should be
		reset or not. It is on by default.
	*/
	public void setResetClassLoader(boolean resetClassLoader)
	{
		this.resetClassLoader = resetClassLoader;
	}
	
	public boolean getResetClassLoader()
	{
		return resetClassLoader;
	}
	
	/**
		This flag controls if the error flag should be
		reset or not. It is on by default.
	*/
	public void setResetErrorFlag(boolean resetErrorFlag)
	{
		this.resetErrorFlag = resetErrorFlag;
	}
	
	public boolean getResetrrorFlag()
	{
		return resetErrorFlag;
	}

	/*
		From the ExtensionsFileProcessor interface
		------------------------------------------
	*/
	@Override
	public void begin(ExtensionsManager manager, int numFiles)
	{
		log.debug("Starting ResetProcessor: errorFlag="+resetErrorFlag + "; classLoader="+resetClassLoader);
		this.numFiles = 0;
	}
	
	@Override
	public void processFile(ExtensionsManager manager, WriteableExtensionsFile wFile)
	{
		ExtensionsFile xtFile = wFile.getExtensionsFile();
		log.debug("Resetting: " + xtFile);
		numFiles++;
		try
		{
			wFile.open();
			if (resetErrorFlag)
			{
				wFile.setError(false);
				results.forgetOldResults(xtFile);
			}
			if (resetClassLoader) 
			{
				wFile.resetClassLoader();
			}
		}
		finally
		{
			wFile.close();
		}
	}

	@Override
	public void done(ExtensionsManager manager)
	{}

	@Override
	public void done(ExtensionsManager manager, Throwable t)
	{}
	// ---------------------------------------------

	/**
		Get the number of files that was processed.
	*/
	public int getNumFiles()
	{
		return numFiles;
	}

	
}
