/**
	$Id$

	Copyright (C) 2015 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.util.listable;


import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import net.sf.basedb.core.DbControl;
import net.sf.basedb.core.HibernateUtil;
import net.sf.basedb.core.Item;
import net.sf.basedb.core.ItemQuery;
import net.sf.basedb.core.Type;
import net.sf.basedb.core.dbengine.DbEngine;

/**
	Abstract base class for source item transformers that can be useful
	for implementing some common functionality.
	
	@author Nicklas
	@since 3.5
*/
public abstract class AbstractSourceItemTransformer 
	implements SourceItemTransformer 
{

	private final Item source;
	private final Item target;
	
	/**
		Create a new instance that transforms between the given
		source and target item types.
	*/
	protected AbstractSourceItemTransformer(Item source, Item target)
	{
		this.source = source;
		this.target = target;
	}
		
	@Override
	public Item getSourceItemType()
	{
		return source;
	}

	@Override
	public Item getTargetItemType() 
	{
		return target;
	}

	/**
		Safely execute the query even if the number of values in the 'values' set
		is higher than {@link DbEngine#getMaxParametersInQuery()}. In this case, the
		values will be divided into subsets and the result of the query is merged
		in memory. It is important that the query is such that executing it multiple
		times with different subsets return the same result as if it was only executed
		a single time with the complete set.
		
		@param query The query to execut
		@param parameterName The name of the parameter to set for the values
		@param values The values to use in the query
		@return The result of the query.idList() method
	*/
	protected Set<Integer> safeIdList(DbControl dc, ItemQuery<?> query, String parameterName, Set<Integer> values)
	{
		Set<Integer> allId = new HashSet<Integer>();
		int numValues = values.size();
		int maxParameters = HibernateUtil.getDbEngine().getMaxParametersInQuery();
		
		if (numValues > maxParameters)
		{
			List<Integer> full = new ArrayList<Integer>(values);
			int startIndex = 0;
			int endIndex = Math.min(maxParameters, numValues);
			while (startIndex < numValues)
			{
				query.setParameter(parameterName, full.subList(startIndex, endIndex), Type.INT);
				allId.addAll(query.idList(dc));
				startIndex = endIndex;
				endIndex = Math.min(startIndex + maxParameters, numValues);
			}
		}
		else if (numValues > 0)
		{
			query.setParameter(parameterName, values, Type.INT);
			allId.addAll(query.idList(dc));
		}
		return allId;
	}

}
