/**
	$Id: PhysicalBioAssayImporter.java 7331 2017-04-10 07:51:04Z nicklas $

	Copyright (C) 2011 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.plugins.batchimport;

import java.util.Arrays;
import java.util.Collections;
import java.util.List;
import java.util.Set;

import net.sf.basedb.core.ArraySlide;
import net.sf.basedb.core.BioMaterialEvent;
import net.sf.basedb.core.BioMaterialEventSource;
import net.sf.basedb.core.DbControl;
import net.sf.basedb.core.Extract;
import net.sf.basedb.core.Hardware;
import net.sf.basedb.core.PhysicalBioAssay;
import net.sf.basedb.core.InvalidDataException;
import net.sf.basedb.core.Item;
import net.sf.basedb.core.ItemQuery;
import net.sf.basedb.core.ItemSubtype;
import net.sf.basedb.core.Kit;
import net.sf.basedb.core.Permission;
import net.sf.basedb.core.PermissionDeniedException;
import net.sf.basedb.core.PluginParameter;
import net.sf.basedb.core.Protocol;
import net.sf.basedb.core.StringParameterType;
import net.sf.basedb.core.plugin.GuiContext;
import net.sf.basedb.util.parser.DefaultItemFunction;
import net.sf.basedb.util.parser.FlatFileParser;
import net.sf.basedb.util.parser.Mapper;

/**
	Plug-in for importing physical bioassays items in a batch. The plug-in can create new 
	items and updated existing items.

	@author nicklas
	@since 3.0
	@base.modified $Date: 2017-04-10 09:51:04 +0200 (må, 10 apr 2017) $
*/
public class PhysicalBioAssayImporter
	extends AbstractItemImporter<PhysicalBioAssay>
{

	private static final Set<GuiContext> guiContexts =  
		Collections.singleton(new GuiContext(Item.PHYSICALBIOASSAY, GuiContext.Type.LIST));
	
	protected static final PluginParameter<String> hardwareColumnMapping = new PluginParameter<String>(
			"hardwareColumnMapping",
			"Hardware",
			"Mapping that picks the name or ID of a hardware from the data columns. " +
			"The plug-in will first try to find an item with the given name. If none is found and " +
			"the value is numeric it will try to load by internal ID. " +
			"Example: \\Hardware\\",
			optionalColumnMapping
			);

	protected static final PluginParameter<String> arraySlideColumnMapping = new PluginParameter<String>(
			"arraySlideColumnMapping",
			"Array slide",
			"Mapping that picks the name or ID of an array slide from the data columns. " +
			"The plug-in will first try to find a slide with the given name. If none is found and " +
			"the value is numeric it will try to load by internal ID. " +
			"Example: \\Array slide\\",
			optionalColumnMapping
			);

	protected static final PluginParameter<String> sizeColumnMapping = 
		new PluginParameter<String>(
				"sizeColumnMapping",
				"Size",
				"Mapping that picks the size of the bioassay. The " +
				"values must be in the numeric format specified by the 'Number format' parameter." +
				"Example: \\Size\\",
				optionalColumnMapping
		);

	protected static final PluginParameter<String> extractPositionColumnMapping = 
		new PluginParameter<String>(
				"extractPositionColumnMapping",
				"Extract position",
				"Mapping that picks the (1-based) position an extract " +
				"was used on. The value should be between 1 and the size of the bioassay and must be " +
				"in the numeric format specified by the 'Number format' parameter." +
				"Example: \\Array index\\",
				optionalColumnMapping
		);
	
	private static final PluginParameter<String> multiplePositionsError = 
		new PluginParameter<String>(
			"multiplePositionsError",
			"Extract on multiple positions",
			"What to do if trying to assign an extract to more than one position " +
			"on the bioassay\n\n"+
			"ignore = Update the position overriding previous settings\n" +
			"fail = Stop with an error message",
			new StringParameterType(255, "fail", false, 1, 0, 0, 
				Arrays.asList( new String[] { "ignore", "fail"} ))
		);

	
	public PhysicalBioAssayImporter()
	{}

	/*
		From the InteractivePlugin interface
		------------------------------------
	*/
	@Override
	public Set<GuiContext> getGuiContexts()
	{
		return guiContexts;
	}
	// ------------------------------------

	/*
		From the AbstractItemImporter class
		------------------------------------
	*/
	
	private Mapper nameMapper;
	private Mapper descriptionMapper;
	private Mapper sizeMapper;
	private Mapper createdMapper;
	private Mapper registeredMapper;
	private Mapper protocolMapper;
	private DefaultItemFunction<Protocol> defaultProtocolFunction;
	private Mapper hardwareMapper;
	private DefaultItemFunction<Hardware> defaultHardwareFunction;
	private Mapper kitMapper;
	private Mapper parentMapper;
	private Mapper usedQuantityMapper;
	private Mapper extractPositionMapper;
	private Mapper arraySlideMapper;
	private boolean failIfMultiplePositions;
	
	/**
		Useable methods are:
		<ul>
		<li>{@link PropertyIdMethod#NAME}
		<li>{@link InternalIdMethod#INTERNAL_ID}
		<li>{@link FallbackIdMethod#NAME_OR_ID}
		</ul>
	*/
	@Override
	protected IdMethod[] getIdMethods()
	{
		return new IdMethod[] { PropertyIdMethod.NAME,  
				InternalIdMethod.INTERNAL_ID, FallbackIdMethod.NAME_OR_ID};
	}
	
	@Override
	protected Item getItemForSubtypes()
	{
		return Item.PHYSICALBIOASSAY;
	}

	/**
		Adds column mappings for name, externalId, description,
		protocol, creation date, original quantity, pooled, parent
		and used quantity from parent.
	*/
	@Override
	protected void addMoreColumnMappingParameters(List<PluginParameter<?>> parameters)
	{
		parameters.add(internalIdColumnMapping);
		parameters.add(nameColumnMapping);
		parameters.add(subtypeColumnMapping);
		parameters.add(descriptionColumnMapping);
		parameters.add(sizeColumnMapping);
		parameters.add(protocolColumnMapping);
		parameters.add(kitColumnMapping);
		parameters.add(hardwareColumnMapping);
		
		parameters.add(SampleImporter.createdColumnMapping);
		parameters.add(registeredColumnMapping);
		parameters.add(SampleImporter.parentColumnMapping);
		parameters.add(SampleImporter.usedQuantityColumnMapping);
		parameters.add(extractPositionColumnMapping);
		parameters.add(arraySlideColumnMapping);
		parameters.add(permissionTemplateColumnMapping);
	}
	
	@Override
	protected void addMoreErrorParameters(List<PluginParameter<?>> parameters)
	{
		parameters.add(multiplePositionsError);
	}
	
	/**
		Calls {@link PhysicalBioAssay#getQuery()}.
	*/
	@Override
	protected ItemQuery<PhysicalBioAssay> createItemQuery()
	{
		return PhysicalBioAssay.getQuery();
	}
	
	@Override
	protected void createColumnMappers(FlatFileParser ffp, boolean cropStrings)
	{
		nameMapper = getMapper(ffp, (String)job.getValue("nameColumnMapping"), 
				cropStrings ? PhysicalBioAssay.MAX_NAME_LENGTH : null, null);
		descriptionMapper = getMapper(ffp, (String)job.getValue("descriptionColumnMapping"), 
				cropStrings ? PhysicalBioAssay.MAX_DESCRIPTION_LENGTH : null, null);
		sizeMapper = getMapper(ffp, (String)job.getValue("sizeColumnMapping"), null, null);
		createdMapper = getMapper(ffp, (String)job.getValue("createdColumnMapping"), null, null);
		registeredMapper = getMapper(ffp, (String)job.getValue("registeredColumnMapping"), null, null);
		defaultProtocolFunction = getDefaultItemFunction(Item.PROTOCOL);
		protocolMapper = getMapper(ffp, (String)job.getValue("protocolColumnMapping"), null, null, defaultProtocolFunction);
		kitMapper = getMapper(ffp, (String)job.getValue("kitColumnMapping"), null, null);
		defaultHardwareFunction = getDefaultItemFunction(Item.HARDWARE);
		hardwareMapper = getMapper(ffp, (String)job.getValue("hardwareColumnMapping"), null, null, defaultHardwareFunction);
		parentMapper = getMapper(ffp, (String)job.getValue("parentColumnMapping"), null, null);
		usedQuantityMapper = getMapper(ffp, (String)job.getValue("usedQuantityColumnMapping"), null, null);
		extractPositionMapper = getMapper(ffp, (String)job.getValue("extractPositionColumnMapping"), null, null);
		arraySlideMapper = getMapper(ffp, (String)job.getValue("arraySlideColumnMapping"), null, null);
		this.failIfMultiplePositions = "fail".equals(getErrorOption("multiplePositionsError"));
	}
	
	@Override
	protected PhysicalBioAssay createItem(DbControl dc, FlatFileParser.Data data)
	{
		PhysicalBioAssay pba = PhysicalBioAssay.getNew(dc);
		if (registeredMapper != null) 
		{
			pba.setEntryDate(parseDate(registeredMapper.getValue(data)));
		}
		updateItem(dc, pba, data);
		return pba;
	}
	
	@Override
	protected void updateItem(DbControl dc, PhysicalBioAssay pba, FlatFileParser.Data data)
	{
		if (nameMapper != null) pba.setName(nameMapper.getValue(data));
		if (descriptionMapper != null) pba.setDescription(descriptionMapper.getValue(data));
		if (sizeMapper != null) pba.setSize(sizeMapper.getInt(data));
		BioMaterialEvent creationEvent = pba.getCreationEvent();
		if (createdMapper != null) creationEvent.setEventDate(parseDate(createdMapper.getValue(data)));
		updateItemSubtype(dc, pba, data);
		if (protocolMapper != null) 
		{
			ItemSubtype type = ItemSubtype.getRelatedSubtype(dc, pba, Item.PROTOCOL, 0);
			defaultProtocolFunction.setCurrentSubtype(type);
			String nameOrId = protocolMapper.getValue(data);
			Protocol protocol = findProtocol(dc, FallbackIdMethod.NAME_OR_EXTERNALID_OR_ID, nameOrId, type);
			if (nameOrId == null || protocol != null) creationEvent.setProtocol(protocol);
		}
		if (kitMapper != null) 
		{
			ItemSubtype type = ItemSubtype.getRelatedSubtype(dc, pba, Item.KIT, 0);
			String nameOrId = kitMapper.getValue(data);
			Kit kit = findKit(dc, FallbackIdMethod.NAME_OR_ID, nameOrId, type);
			if (nameOrId == null || kit != null) creationEvent.setKit(kit);
		}
		if (hardwareMapper != null)
		{
			ItemSubtype type = ItemSubtype.getRelatedSubtype(dc, pba, Item.HARDWARE, 0);
			defaultHardwareFunction.setCurrentSubtype(type);
			String nameOrId = hardwareMapper.getValue(data);
			Hardware hardware = findHardware(dc, FallbackIdMethod.NAME_OR_ID, nameOrId, type);
			if (nameOrId == null || hardware != null) creationEvent.setHardware(hardware);
		}
		if (arraySlideMapper != null)
		{
			String nameOrId = arraySlideMapper.getValue(data);
			ArraySlide slide = findArraySlide(dc, FallbackIdMethod.NAME_OR_BARCODE_OR_ID, nameOrId);
			if (slide != null && slide.isUsedByOther(pba))
			{
				if ("fail".equals(getErrorOption("noUsePermissionToReferenceError")))
				{
					throw new PermissionDeniedException(Permission.USE, 
						slide + ". It is already used by another bioassay.");
				}
			}
			else if (nameOrId == null || slide != null) 
			{
				pba.setArraySlide(slide);
				if (slide != null && sizeMapper == null)
				{
					// If the size hasn't been mapped we try to get the
					// value from the array design
					try
					{
						pba.setSize(slide.getArrayBatch().getArrayDesign().getNumArrays());
					}
					catch (RuntimeException ex)
					{}
				}
			}
		}
		updateMultiLineItem(dc, pba, data, 0);
	}
	
	/**
		Add parent extracts and the used quantities
	*/
	@Override
	protected void updateMultiLineItem(DbControl dc, PhysicalBioAssay pba, 
		FlatFileParser.Data data, int multiLineNum)
	{
		if (parentMapper != null)
		{
			String nameOrId = parentMapper.getValue(data);
			ItemSubtype type = ItemSubtype.getRelatedSubtype(dc, pba, Item.EXTRACT, 0);
			Extract parent = findExtract(dc, FallbackIdMethod.NAME_OR_EXTERNALID_OR_ID, nameOrId, type);
			if (parent != null)
			{
				BioMaterialEvent creationEvent = pba.getCreationEvent();
				BioMaterialEventSource eventSource = creationEvent.getEventSource(parent);
				int currentPosition = 0;
				if (eventSource == null)
				{
					eventSource = creationEvent.addSource(parent);
				}
				else
				{
					currentPosition = eventSource.getPosition();
				}
				if (usedQuantityMapper != null)
				{
					eventSource.setUsedQuantity(usedQuantityMapper.getFloat(data));
				}
				if (extractPositionMapper != null)
				{
					Integer position = extractPositionMapper.getInt(data);
					if (position != null) 
					{
						if (currentPosition != 0 && position != currentPosition && failIfMultiplePositions)
						{
							throw new InvalidDataException("Extract '" + parent.getName() + 
									"' is already associated with position " + currentPosition + 
									" on bioassay '" + pba.getName() + "'. It is not possible to " +
									"also associate it with position " + position);
						}
						eventSource.setPosition(position);
					}
				}
			}
		}
	}
	// ------------------------------------
}
