/**
	$Id: DefaultConfigurationValues.java 6459 2014-05-27 08:46:05Z nicklas $

	Copyright (C) 2011 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.plugins.gtf;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import net.sf.basedb.core.BaseException;
import net.sf.basedb.core.ExtendedProperties;
import net.sf.basedb.core.InvalidDataException;
import net.sf.basedb.core.Job;
import net.sf.basedb.core.ParameterType;
import net.sf.basedb.core.Permission;
import net.sf.basedb.core.PermissionDeniedException;
import net.sf.basedb.core.PluginConfiguration;
import net.sf.basedb.core.PluginDefinition;
import net.sf.basedb.core.plugin.ParameterValues;

/**
	Provides a default configuration values for GTF file parsing 
	when none has been provided by the user.
	
	@author nicklas
	@since 3.0
*/
class DefaultConfigurationValues
	implements ParameterValues
{

	private final ParameterValues configuration;
	private Map<String, Object> defaultValues;
	
	DefaultConfigurationValues(ParameterValues configuration)
	{
		this.configuration = configuration;
	}
	
	@Override
	public PluginDefinition getPluginDefinition()
	{
		return configuration == null ? null : configuration.getPluginDefinition();
	}

	@Override
	public Job getJob()
	{
		return configuration == null ? null : configuration.getJob();
	}


	@Override
	public PluginConfiguration getPluginConfiguration()
	{
		return configuration == null ? null : configuration.getPluginConfiguration();
	}


	@Override
	public int getId()
	{
		return configuration == null ? 0 : configuration.getId();
	}

	@Override
	public Object getValue(String name)
		throws PermissionDeniedException, BaseException
	{
		Object value = configuration == null ? null : configuration.getValue(name);
		if (value == null)
		{
			if (defaultValues == null)
			{
				defaultValues = new HashMap<String, Object>();
				defaultValues.put("dataSplitterRegexp", "\\t");
				defaultValues.put("dataHeaderRegexp", "<seqname>\\t.*<transcript_id>.*");
				defaultValues.put("ignoreRegexp", "^#.*");
				defaultValues.put("minDataColumns", 4);
				defaultValues.put("complexExpressions", "allow");
				
				// Reporter importer mappings
				defaultValues.put("reporterIdColumnMapping", "\\<transcript_id>\\@\\<seqname>\\");
				defaultValues.put("nameColumnMapping", "\\<transcript_id>\\@\\<seqname>\\");
				defaultValues.put("symbolColumnMapping", "\\<gene_id>\\");
				if (ExtendedProperties.getProperty("ReporterData", "chromosome") != null)
				{
					defaultValues.put("extendedColumnMapping.chromosome", "\\<seqname>\\");
				}

				// Reporter map importer mappings (also use reporterIdColumnMapping)
				defaultValues.put("featureIdColumnMapping", "\\<transcript_id>\\@\\<seqname>\\");
			}
			value = defaultValues.get(name);
		}
		return value;
	}

	@Override
	public List<?> getValues(String name)
		throws PermissionDeniedException, BaseException
	{
		return configuration == null ? null : configuration.getValues(name);
	}

	@Override
	public <T> void setValue(String name, ParameterType<T> type, T value)
		throws PermissionDeniedException, InvalidDataException, BaseException
	{
		if (configuration != null)
		{
			configuration.setValue(name, type, value);
		}
		else
		{
			throw new PermissionDeniedException(Permission.WRITE, "Parameter: "+name);
		}
	}

	@Override
	public <T> void setValues(String name, ParameterType<T> type, List<T> values)
		throws PermissionDeniedException, InvalidDataException, BaseException
	{
		if (configuration != null)
		{
			configuration.setValues(name, type, values);
		}
		else
		{
			throw new PermissionDeniedException(Permission.WRITE, "Parameter: "+name);
		}
	}
	
}