/*
	$Id: TestTag.java 6082 2012-08-14 13:54:00Z nicklas $

	Copyright (C) 2005 Nicklas Nordborg
	Copyright (C) 2006 Jari Häkkinen, Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
import net.sf.basedb.core.*;
import net.sf.basedb.core.query.Expressions;
import net.sf.basedb.core.query.Hql;
import net.sf.basedb.core.query.Restrictions;

import java.util.Date;
import java.util.Set;

public class TestTag
{

	static boolean ok = true;
	public static void main(String[] args)
	{
		TestUtil.checkArgs(args);
		TestUtil.begin();
		ok = test_all();
		TestUtil.stop();
	}

	static boolean test_all()
	{
		write("++Testing tags");
		write_header();
		// Standard tests: create, load, list
		int id = test_create("Test label #1", 0, true);
		int id2 = test_create("Test label #2", 0, false);
		test_load(id);
		test_list(-1);

		// Standard test: Delete
		if (TestUtil.waitBeforeDelete()) TestUtil.waitForEnter();
		test_delete(id);
		test_delete(id2);
		write("++Testing tags "+(ok ? "OK" : "Failed")+"\n");
		return ok;
	}

	static int test_create(String name, int subtypeId, boolean setAll)
	{
		if (!TestUtil.hasPermission(Permission.CREATE, Item.TAG)) return 0;
		int id = 0;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			Tag t = Tag.getNew(dc);
			t.setName(name);
			if (subtypeId != 0)
			{
				t.setItemSubtype(ItemSubtype.getById(dc, subtypeId));
			}
			if (setAll)
			{
				t.setDescription("Added at "+new Date());
			}
			dc.saveItem(t);
			dc.commit();
			id = t.getId();
			dc = TestUtil.getDbControl();
			dc.reattachItem(t, false);
			write_item(0, t);
			write("--Create tag OK");
		}
		catch (Throwable ex)
		{
			write("--Create tag FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
		return id;
	}

	static void test_load(int id)
	{
		if (id == 0) return;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			Tag t = Tag.getById(dc, id);
			write_item(0, t);
			write("--Load tag OK");
		}
		catch (Throwable ex)
		{
			write("--Load tag FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}

	static int test_load(String name)
	{
		if (name == null) return 0;
		DbControl dc = null;
		int id = 0;
		try
		{
			dc = TestUtil.getDbControl();
			ItemQuery<Tag> query = Tag.getQuery();
			query.restrict(Restrictions.eq(Hql.property("name"), Expressions.string(name)));
			Tag t = query.list(dc).get(0);
			write_item(0, t);
			write("--Load tag by name OK");
			id = t.getId();
		}
		catch (Throwable ex)
		{
			write("--Load tag by name FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
		return id;
	}

	
	static void test_list(int expectedResults)
	{
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			ItemResultList<Tag> l = Tag.getQuery().list(dc);
			for (int i = 0; i<l.size(); i++)
			{
				write_item(i, l.get(i));
			}
			if (expectedResults >= 0 && expectedResults != l.size())
			{
				throw new BaseException("Expected "+expectedResults+" results, not "+l.size());
			}
			write("--List tags OK ("+l.size()+")");
		}
		catch (Throwable ex)
		{
			write("--List tags FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}

	static void test_delete(int id)
	{
		if (id == 0) return;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			Tag t = Tag.getById(dc, id);
			dc.deleteItem(t);
			Set<ItemProxy> using = t.getUsingItems();
			if (using.size() > 0) 
			{
			   throw new BaseException(t + " is used by " + using);
			}
			dc.commit();
			write("--Delete tag OK");
		}
		catch (Throwable ex)
		{
			write("--Delete tag FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}

	static void write_header()
	{
		if (!TestUtil.getSilent())
		{
			write("   \tID \tName      \tSubtype\tDescription");
			write("-- \t-- \t--------- \t-------\t-----------");
		}
	}
	static void write_item(int i, Tag t)
		throws BaseException
	{
		if (!TestUtil.getSilent()) System.out.println(i+":\t"+t.getId()+"\t"+t.getName()+"\t"+t.getItemSubtype()+"\t"+t.getDescription());
	}
	static void write(String message)
	{
		System.out.println(message);
	}
}
